/*
 * Copyright 2017 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.vms.call.script.js;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.mozilla.javascript.NativeArray;
import org.mozilla.javascript.NativeObject;

/**
 * Class Util contains various utility methods for use with JavaScript.
 */
public class Util {
	/**
	 * Convert JavaScript string array to Java string array.
	 * @param na JavaScript string array
	 * @return Java string array
	 */
	public static String[] convertStringArray(NativeArray na) {
		String[] res = new String[na.size()];
		for(int i = 0; i < res.length; i++) {
			res[i] = (String)na.get(i);
		}
		return res;
	}
	/**
	 * Class MyNativeObject is an extension of builtin NativeObject to provide a default value.
	 *
	 */
	private static class MyNativeObject extends NativeObject {
		private static final long serialVersionUID = 1L;
		private String defval;
		/**
		 * Construct instance.
		 * @param defval the default value to be used
		 */
		public MyNativeObject(String defval) {
			this.defval = defval;
		}
		@Override
		public Object getDefaultValue(Class<?> clz) {
			return defval;
		}
	}
	/**
	 * Convert Java Map to JavaScript object.
	 * @param <T> type of values
	 * @param map Java map
	 * @return JavaScript object
	 */
	public static <T> NativeObject convertMap(Map<String,T> map) {
		NativeObject o = new MyNativeObject(map.toString());
		Iterator<Map.Entry<String,T>> it = map.entrySet().iterator();
		while(it.hasNext()) {
			Map.Entry<String,T> ent = it.next();
			o.put(ent.getKey(), o, ent.getValue());
		}
		return o;
	}
	/**
	 * Class MyNativeArray is an extension of builtin NativeArray to provide a default value.
	 */
	public static class MyNativeArray extends NativeArray {
		private static final long serialVersionUID = 1L;
		private String defval;
		/**
		 * Construct instance.
		 * @param oa Java array
		 * @param defval the default value to be used
		 */
		public MyNativeArray(Object[] oa, String defval) {
			super(oa);
			this.defval = defval;
		}
		@Override
		public Object getDefaultValue(Class<?> clz) {
			return defval;
		}
	}
	/**
	 * Convert Java List to JavaScript array.
	 * @param <T> type of values
	 * @param lst Java List
	 * @return JavaScript array
	 */
	public static <T> NativeArray convertList(List<T> lst) {
		return new MyNativeArray(lst.toArray(), lst.toString());
	}
	/**
	 * Convert Java List of Map to JavaScript array of objects.
	 * @param <T> type of values
	 * @param lst Java List of Map
	 * @return JavaScript array of objects
	 */
	public static <T> NativeArray convertListMap(List<Map<String,T>> lst) {
	    Object[] temp = new Object[lst.size()];
		for(int i = 0; i < lst.size(); i++) {
			temp[i] = convertMap(lst.get(i));
		}
		return new MyNativeArray(temp, lst.toString());
	}
}
